searchData={"items":[{"type":"module","title":"Heyya","doc":"Heyya is here to help with you testing. It provides a\nset of ExUnit CaseTemplate's that make it easier to test Phoenix\nComponents, LiveViews, and LiveComponent's.","ref":"Heyya.html"},{"type":"module","title":"Heyya.LiveCase","doc":"`Heyya.LiveCase` module provides helper methods that make it easier to write a\nmore linear live view test. It takes care of the fact that there's\nconnection state, view state, and last html state.\n\n\nIt does this by creating a `LiveTestSession` struct via `start/2` that\nholds the connection, view, and html in a single struct. This is\nwrapped in an CaseTemplate so that you can use it in your tests.\n\nThese ideas origionally came from: https://www.reddit.com/r/elixir/comments/ydyt2m/better_liveview_tests/\n\nThis implementation doesn't hide follow (in large part because\nof the ergonomics of follow_redirect being a macro)","ref":"Heyya.LiveCase.html"},{"type":"module","title":"Example - Heyya.LiveCase","doc":"```\ndefmodule MyPhoenixWeb.ListLiveTest do\n  use Heyya.LiveCase\n  use MyPhoenixWeb.ConnCase\n\n  test \"widget list with new button\", %{conn: conn} do\n    start(conn, ~p|/widgets|)\n    |> assert_html(\"Widgets List\")\n    |> click(\"a\", \"New Widgets\")\n    |> follow(~p|/widgets/new|)\n  end\nend\n```","ref":"Heyya.LiveCase.html#module-example"},{"type":"function","title":"Heyya.LiveCase.assert_element/3","doc":"","ref":"Heyya.LiveCase.html#assert_element/3"},{"type":"function","title":"Heyya.LiveCase.assert_html/2","doc":"Asserts that the html in the LiveTestSession matches the\nprovided expected html.\n\nReturns the LiveTestSession.","ref":"Heyya.LiveCase.html#assert_html/2"},{"type":"macro","title":"Heyya.LiveCase.assert_matches_snapshot/2","doc":"This macro will assert that the rendered html matches the snapshot.","ref":"Heyya.LiveCase.html#assert_matches_snapshot/2"},{"type":"macro","title":"Options - Heyya.LiveCase.assert_matches_snapshot/2","doc":"The macro accepts the following options in a keyword list:\n\n- `:name` - The name of the snapshot. Defaults to the\nline number. It's not recommended to use the default since\nmoving the test will change the snapshot name.\nNote: names are scoped per function.\n\n- `:selector` - The selector to use to match\nthe snapshot. Defaults to \"main\". Selector should\nalways select a single element so that LiveViewTest can\nrender it to html.","ref":"Heyya.LiveCase.html#assert_matches_snapshot/2-options"},{"type":"macro","title":"Example - Heyya.LiveCase.assert_matches_snapshot/2","doc":"For example the following will start the live view at /numbers and\nthen assert that the element selected by #btn-32 matches\nthe snapshot named \"numbers\"\n\n```elixir\ntest \"/numbers renders the a button\", %{conn: conn} do\n  conn\n  |> start(~p\"/numbers\")\n  |> assert_matches_snapshot(name: \"numbers\", selector: \"#btn-32\")\nend\n```","ref":"Heyya.LiveCase.html#assert_matches_snapshot/2-example"},{"type":"function","title":"Heyya.LiveCase.assert_page_title/2","doc":"","ref":"Heyya.LiveCase.html#assert_page_title/2"},{"type":"function","title":"Heyya.LiveCase.await_async/2","doc":"This method will use LiveViewTest.render_async to ensure that all\noutstanding async operations are completed.  By default this\nwill wait the ExUnit default timeout.","ref":"Heyya.LiveCase.html#await_async/2"},{"type":"function","title":"Heyya.LiveCase.click/3","doc":"This method clicks on an element that's selected via selector then renders\nthe result. Finally returning a new LiveTestSession with updated rendered html\n\nThis will also assert that there is an element that's specified by the selector\n\nThis doesn't follow navigations or redirect.\n\nreturns: `%Heyya.LiveTestSession{}`","ref":"Heyya.LiveCase.html#click/3"},{"type":"function","title":"Heyya.LiveCase.focus/2","doc":"","ref":"Heyya.LiveCase.html#focus/2"},{"type":"macro","title":"Heyya.LiveCase.follow/2","doc":"Macro that creates code to follow a redirect. This takes\nin a LiveTestSession and will assert that a redirect exists.","ref":"Heyya.LiveCase.html#follow/2"},{"type":"function","title":"Heyya.LiveCase.form/3","doc":"Sets the form specified by the selector to a new value.\n\nThis doesn't submit the form\n\nreturns: `%Heyya.LiveTestSession{}`","ref":"Heyya.LiveCase.html#form/3"},{"type":"function","title":"Heyya.LiveCase.refute_element/3","doc":"","ref":"Heyya.LiveCase.html#refute_element/3"},{"type":"function","title":"Heyya.LiveCase.refute_html/2","doc":"Refute that the html in the LiveTestSession matches the provided html string.\n\nReturns the LiveTestSession.","ref":"Heyya.LiveCase.html#refute_html/2"},{"type":"function","title":"Heyya.LiveCase.refute_page_title/2","doc":"","ref":"Heyya.LiveCase.html#refute_page_title/2"},{"type":"macro","title":"Heyya.LiveCase.start/2","doc":"Macro that creates code to start a new LiveTestSession. This takes in a plug\nconnection (Usually from your ConnTest)","ref":"Heyya.LiveCase.html#start/2"},{"type":"function","title":"Heyya.LiveCase.submit_form/3","doc":"Submits the form and renders the result of submitting.\n\nThis doesn't follow any redirects.\n\nreturns: `%Heyya.LiveTestSession{}`","ref":"Heyya.LiveCase.html#submit_form/3"},{"type":"module","title":"Heyya.LiveComponentCase","doc":"A test case template that provides helpers\n  for testing live components. It relies on LiveComponentHost\n  being mounted.\n\n  It allows for testing dyanmic content such as\n  live components with a simplified stack.","ref":"Heyya.LiveComponentCase.html"},{"type":"module","title":"Usage - Heyya.LiveComponentCase","doc":"```elixir\n  # component_test.exs\n  use Heyya.LiveComponentCase\n  use ExampleWeb.ConnCase\n\n  # By default LiveComponentCase will look for a render function in the module\n  def render(assigns) do\n    ~H|<.live_component module={ExampleWeb.LiveCounterComponent} id=\"example\" />|\n  end\n\n  test \"Test Counter\", %{conn: conn} do\n    conn\n    |> start()\n    |> click(\"button.increment\")\n    |> assert_html(\"Counter: 1\")\n  end\n  ```","ref":"Heyya.LiveComponentCase.html#module-usage"},{"type":"module","title":"Options - Heyya.LiveComponentCase","doc":"- `:view_module` - The module that will render the live component\n  - `:view_method` - The method that will render the live component\n  - `:base_path` - The path to the live component host. Defaults to \"/dev/heyya/host\"\n\n  ## `Heyya.LiveComponentHost`\n\n  The `Heyya.LiveComponentHost` is a live view that renders only a single live\n  component with no other layout or content. It is used to test live components in\n  isolation of so don't mount it in scope with complex Plugs.\n\n  ```elixir\n  if Enum.member?([:dev, :test], Mix.env()) do\n    scope \"/dev\" do\n      pipe_through :browser\n      live \"/heyya/host\", Heyya.LiveComponentHost\n    end\n  end\n  ```","ref":"Heyya.LiveComponentCase.html#module-options"},{"type":"macro","title":"Heyya.LiveComponentCase.start/1","doc":"","ref":"Heyya.LiveComponentCase.html#start/1"},{"type":"module","title":"Heyya.LiveComponentHost","doc":"This live view takes in a Module name and a\nmethod name then dynamically renders it. Params\n(other than module anme and method name) are\npassed along to the function as assigns.\n\nContent is wrapped in a div with\nid \"heyya-inner-content\" so that we know what\noutter chrome can be ignored.","ref":"Heyya.LiveComponentHost.html"},{"type":"function","title":"Heyya.LiveComponentHost.empty_layout/1","doc":"","ref":"Heyya.LiveComponentHost.html#empty_layout/1"},{"type":"module","title":"Heyya.LiveTestSession","doc":"This module is a struct to hold the state of a session of live view testing.\n\nIt wraps up Plug connection, the current view, and the latest rendered html","ref":"Heyya.LiveTestSession.html"},{"type":"type","title":"Heyya.LiveTestSession.t/0","doc":"","ref":"Heyya.LiveTestSession.html#t:t/0"},{"type":"module","title":"Heyya.SnapshotCase","doc":"`Heyya.SnapshotCase` allows for fast snapshot\ntesting of Phoenix components. Snapshot testing\ncomponents is a fast and easy way to ensure that\nthey work and produce what they expected to\nproduce without having to hand write\nassertions.","ref":"Heyya.SnapshotCase.html"},{"type":"module","title":"Complex Tests Made Easy - Heyya.SnapshotCase","doc":"Suppose you have a component button with a color\nand and icon. You want to test that the correct\ncss class is applied to the whole dom tree. Instead\nassert that the dom tree is functionally equivalent\nto the expected output.\n\n```elixir\ncomponent_snapshot_test \"Eiffel 65\" do\n  assigns = %{}\n\n  ~H|<.button phx-click=\"click\" class=\"bg-blue-500\">Click me</.button>|\nend\n```","ref":"Heyya.SnapshotCase.html#module-complex-tests-made-easy"},{"type":"module","title":"Change Tests Faster - Heyya.SnapshotCase","doc":"Any changes to that would require changes to the\ntest are easily updated by running the tests and\nupdating the snapshots. This happens by setting the\nenvironment variable `HEYYA_OVERRIDE` to `true` or `1`.\n\n```\nHEYYA_OVERRIDE=true mix test\n```","ref":"Heyya.SnapshotCase.html#module-change-tests-faster"},{"type":"macro","title":"Heyya.SnapshotCase.__using__/1","doc":"Wire up the module to prepare for snapshot testing.","ref":"Heyya.SnapshotCase.html#__using__/1"},{"type":"macro","title":"Heyya.SnapshotCase.component_snapshot_test/2","doc":"A named component snapshot test","ref":"Heyya.SnapshotCase.html#component_snapshot_test/2"},{"type":"macro","title":"Heyya.SnapshotCase.component_snapshot_test/3","doc":"A named component snapshot test, where context is passed through.","ref":"Heyya.SnapshotCase.html#component_snapshot_test/3"},{"type":"macro","title":"Heyya.SnapshotCase.inner_test/1","doc":"","ref":"Heyya.SnapshotCase.html#inner_test/1"},{"type":"module","title":"Heyya.SnapshotUtil","doc":"`Heyya.SnapshotUtil` module provides helper methods that make it easier to write snapshot tests.","ref":"Heyya.SnapshotUtil.html"},{"type":"function","title":"Heyya.SnapshotUtil.compare_html/2","doc":"","ref":"Heyya.SnapshotUtil.html#compare_html/2"},{"type":"function","title":"Heyya.SnapshotUtil.directory/1","doc":"Gets the directory to store snapshot files for the test.","ref":"Heyya.SnapshotUtil.html#directory/1"},{"type":"function","title":"Parameters - Heyya.SnapshotUtil.directory/1","doc":"- env: The macro environment which contains metadata about the test file.","ref":"Heyya.SnapshotUtil.html#directory/1-parameters"},{"type":"function","title":"Returns - Heyya.SnapshotUtil.directory/1","doc":"The directory path as a binary where snapshots should be stored.","ref":"Heyya.SnapshotUtil.html#directory/1-returns"},{"type":"function","title":"Heyya.SnapshotUtil.filename/2","doc":"","ref":"Heyya.SnapshotUtil.html#filename/2"},{"type":"function","title":"Heyya.SnapshotUtil.get_snapshot/1","doc":"Gets the stored snapshot for the given macro environment.\n\nThis looks up the snapshot file path and name based on the environment,\nreads the file contents, and returns the snapshot.\n\nIf there is no snapshot file yet, returns an empty binary.","ref":"Heyya.SnapshotUtil.html#get_snapshot/1"},{"type":"function","title":"Parameters - Heyya.SnapshotUtil.get_snapshot/1","doc":"- env: The macro environment which contains metadata about the test module and function.","ref":"Heyya.SnapshotUtil.html#get_snapshot/1-parameters"},{"type":"function","title":"Returns - Heyya.SnapshotUtil.get_snapshot/1","doc":"The stored snapshot as a binary, or empty binary if no snapshot exists.","ref":"Heyya.SnapshotUtil.html#get_snapshot/1-returns"},{"type":"function","title":"Heyya.SnapshotUtil.override?/0","doc":"Should the snapshot be overridden? if it doesn't match.\n\n# Returns\n\nTrue if the snapshot should be overridden, denoted by the environment variable `HEYYA_OVERRIDE`. The value can be `true`, `1`, `yes`, or `YES`.\nFalse if the snapshot should not be overridden","ref":"Heyya.SnapshotUtil.html#override?/0"},{"type":"function","title":"Heyya.SnapshotUtil.overwrite!/3","doc":"","ref":"Heyya.SnapshotUtil.html#overwrite!/3"},{"type":"extras","title":"Heyya","doc":"# Heyya\n\n`Heyya` is a utility to help with testing your [Phoenix](https://www.phoenixframework.org/) components and live view.","ref":"readme.html"},{"type":"extras","title":"Getting Started - Heyya","doc":"Getting Started\n\nTo use Heyya in your Phoenix project, add it to your list of dependencies in mix.exs:\n\n```elixir\ndef deps do\n  [\n    {:heyya, \"~> 1.0.0\"}\n  ]\nend\n```\n\nThen, run `mix deps.get` to install Heyya and its dependencies. Use some case templates provided that make testing easier and more automated.","ref":"readme.html#getting-started"},{"type":"extras","title":"Examples - Heyya","doc":"In order to show how the different tests can be\nimplemented we have an example Phoenix project in the example\nproject.\n\nChanges of note are:\n\n- Added two example live views that show simple pages.\n- Added the Heyya dependency\n- Added snapshot component tests that show variants get expected classes and aggregate components combine as expected.\n- Added live view tests that show Heyya's full page live view testing utilities. Open a page and surf around.\n- Added the `/dev/heyya/host` route for dev and test environments\n- Added a live_component tests that shows stateful component testing with dynamic content without full page data load.","ref":"readme.html#examples"},{"type":"extras","title":"Example Component Snapshot Usage - Heyya","doc":"```\nuse Heyya.SnapshotCase\n\ncomponent_snapshot_test \"Super Simple H1 Test\" do\n  assigns = %{}\n\n  ~H\"\"\"\n   Testing \n  \"\"\"\nend\n```\n\nTo run the snapshot tests, run `mix test` as usual. This will compare the snapshots to the current rendered output of the components and fail the tests if the snapshots do not match.\n\nIf you need to update the snapshots for any reason, you can run `HEYYA_OVERRIDE=true mix test` to reset the snapshot values to the current rendered output of the components.\n\nCredit:\n\nThe initial snapshot code used [Snapshy](https://hex.pm/packages/snapshy). We have since moved on to inspect the html rather than using snapshy. Thank you to them for the initial implementation.","ref":"readme.html#example-component-snapshot-usage"},{"type":"extras","title":"Example Live View Test - Heyya","doc":"```\nuse Heyya.LiveCase\nuse MyPhoenixWeb.ConnCase\n\ntest \"widget list with new button\", %{conn: conn} do\n  start(conn, ~p|/widgets|)\n  |> assert_html(\"Widgets List\")\n  |> click(\"a\", \"New Widgets\")\n  |> follow(~p|/widgets/new|)\nend\n```","ref":"readme.html#example-live-view-test"},{"type":"extras","title":"Example Live View Snapshot Test - Heyya","doc":"```elixir\ntest \"/numbers renders the live_view\", %{conn: conn} do\n  conn\n  |> start(~p\"/numbers\")\n  |> assert_matches_snapshot(name: \"full_view\", selector: \"main\")\nend\n```\n\nThat will start a live view test session and assert that the rendered output of the \nlive view matches the snapshot named \"full_view\" after asserting that there \nis an element with the selector \"main\".\n\nThis is a very fast way to assert that the live view is rendering the expected \noutput. The default selector is \"main\" if none is provided, and you will likely \nwant to provide a selector to ensure that the snapshot is only capturing the \nrelevant part of the rendered output.","ref":"readme.html#example-live-view-snapshot-test"},{"type":"extras","title":"Example Live Component Test - Heyya","doc":"In order to test a live view component we need a full\nendpoint. Attach the component host into the Router for test\nenvironments (attaching for debug is nice sometimes too).\n\nThat can be done like this:\n```\nif Enum.member?([:dev, :test], Mix.env()) do\n  scope \"/dev\" do\n    pipe_through :browser\n\n    live \"/heyya/host\", Heyya.LiveComponentHost\n  end\nend\n```\n\nFrom then on `/dev/heyya/host` will host dynamic content with no layout other than a single wrapper div.\n\n\n```\n  use Heyya.LiveComponentCase\n  use ExampleWeb.ConnCase\n  use Phoenix.Component\n\n  def render(assigns) do\n    ~H\"\"\"\n    <.live_component module={ExampleWeb.LiveCounterComponent} id=\"example\" />\n    \"\"\"\n  end\n\n  test \"Test Counter\", %{conn: conn} do\n    conn\n    |> start()\n    |> click(\"button.increment\")\n    |> assert_html(\"Counter: 1\")\n  end\n```","ref":"readme.html#example-live-component-test"},{"type":"extras","title":"Contributing - Heyya","doc":"We welcome contributions to Heyya! Please see the CONTRIBUTING.md file for guidelines on how to contribute.","ref":"readme.html#contributing"},{"type":"extras","title":"License - Heyya","doc":"Heyya is released under the Apache 2.0","ref":"readme.html#license"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"[]\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright [yyyy] [name of copyright owner]\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.1"}}